#----------------
# 関数の定義
#----------------
# ベクトルの０クリア
def vec(vname, rmax, inval=0):
    vname = [number for number in range(int(rmax))]
    for i in range(int(rmax)):
        vname[int(i)] = inval
    return vname

# マトリクスの０クリア
def mat(mname, rmax1, rmax2, inval=0):
    mname = [number for number in range(int(rmax1))]
    for i in range(int(rmax1)):
        mname[int(i)] = [number for number in range(int(rmax2))]
        for j in range(int(rmax2)):
            mname[int(i)][int(j)] = inval
    return mname

#連立方程式の解法
def solveSE(A,B,n):
    n1 = n - 1
    if n1 == 0 :
        B[0]=B[0]/A[0][0]
        return B
    for k in range(n1):
        k1 = k + 1
        s = A[k][k]
        for j in range(k1,n):
            A[k][j] = A[k][j]/s
        B[k] = B[k]/s
        for i in range(k1,n):
            c = A[i][k]
            for j in range(k1,n):
                A[i][j] = A[i][j] - c*A[k][j]
            B[i] = B[i] - c*B[k]
    B[n-1] = B[n-1]/A[n-1][n-1]
    for m in range(n1):
        k = (n-1) - (m+1)
        k1 = k + 1
        for j in range(k1,n):
            B[k] = B[k] - A[k][j]*B[j]
    return B

#----------------
# データ入力
#----------------

#地震加速度データの読み込み
ydd = []
ydd = vec(ydd,3000,0.0)
file = open('ElCentroNS.txt','r')
dteq = 0.02                #地震加速度の時間刻み
for i in range(2689):
    dd = file.readline()
    ydd[i] = float(dd)
file.close()

#層数の入力
Nn = input("N = ")
Nn = int(Nn)

mi = []
mi = vec(mi,Nn,0.0)        # 各層の質量
ki = []
ki = vec(ki,Nn,0.0)        # 各層の剛性
ms = []
ms = mat(ms,Nn,Nn,0.0)    # 質量マトリクス
ks = []
ks = mat(ks,Nn,Nn,0.0)    # 剛性マトリクス
cs = []
cs = mat(cs,Nn,Nn,0.0)    # 減衰マトリクス
kb = []
kb = mat(kb,Nn,Nn,0.0)    # 剛性方程式の剛性マトリクス
fb = []
fb = vec(fb,Nn,0.0)       # 剛性方程式の外力ベクトル
dx = []
dx = vec(dx,Nn,0.0)       # 変位増分ベクトル
xn = []
xn = vec(xn,Nn,0.0)       # 変位ベクトル
xdn = []
xdn = vec(xdn,Nn,0.0)     # 速度ベクトル
xddn = []
xddn = vec(xddn,Nn,0.0)   # 加速度ベクトル

for i in range(Nn):
    print('No.=',i+1)
    #質量の入力
    d1 = input("m = ")
    mi[i] = float(d1)
    #剛性の入力
    d2 = input("k = ")
    ki[i] = float(d2)

print(ms)
print(type(ms[0][0]))

#減衰定数の入力
hc = input("2h1/om = ")
hc = float(hc)
#解析時間の入力
tm = input("tmax(sec) = ")
tm = float(tm)

# --- 質量マトリクス[M]
for i in range(Nn):
    ms[i][i] = mi[i]
# --- 剛性マトリクス[K]
for i in range(Nn-1):
    ks[i][i] = ki[i] + ki[i+1]
    ks[i][i+1] = -ki[i+1]
    ks[i+1][i] = -ki[i+1]
ks[Nn-1][Nn-1] = ki[Nn-1]
# --- 減衰マトリクス[C]
for i in range(Nn):
    for j in range(Nn):
        cs[i][j] = hc*ks[i][j]

# ------------------------------------------------------------
#     応答解析（平均加速度法）
# ------------------------------------------------------------
fileout = open('output4.txt','w')

tn = 0.0     #時間
dt = dteq    #時間増分
accn = xddn[Nn-1] + ydd[0]

s = str('%10.3f' % tn) + str('  %10.3e' % xn[Nn-1]) + str('  %10.3e' % xdn[Nn-1]) + str('  %10.3e' % accn)
fileout.write(s)
fileout.write('\n')

for n in range(2688):
    tn1 = tn + dt     #tn+1
    if tn1 > tm + dt/2.0 : break
    dydd = ydd[n + 1] - ydd[n]
    # ---- [kb]
    for i in range(Nn):
        for j in range(Nn):
            kb[i][j] = ks[i][j] + (2.0/dt)*cs[i][j] + (4.0/dt**2)*ms[i][j]
    # ---- [fb]
    for i in range(Nn):
        fb[i] = 0.0
        for j in range(Nn):
            fb[i] = fb[i] + ms[i][j]*(-dydd + (4.0/dt)*xdn[j] + 2.0*xddn[j]) + 2.0*cs[i][j]*xdn[j]
    # ---- 連立方程式
    dx = solveSE(kb, fb, Nn)

    for i in range(Nn):
        dxi = dx[i]
        dxdi = (2.0/dt)*dxi - 2.0*xdn[i]
        dxddi = (4.0/dt**2)*dxi - (4.0/dt)*xdn[i] - 2.0*xddn[i]
        xn[i] = xn[i] + dxi
        xdn[i] = xdn[i] + dxdi
        xddn[i] = xddn[i] + dxddi

    accn = xddn[Nn-1] + ydd[n+1]
    s = str('%10.3f' % tn1) + str('  %10.3e' % xn[Nn-1]) + str('  %10.3e' % xdn[Nn-1]) + str('  %10.3e' % accn)
    fileout.write(s)
    fileout.write('\n')
    tn = tn1    # 時間の更新

fileout.close()